import 'package:flutter/services.dart';
import 'package:login_firebase/business/bloc/core/ModBloc.dart';
import 'package:login_firebase/business/dto/ModSessionDTO.dart';
import 'package:login_firebase/business/repo/ISessionRepo.dart';
import 'package:login_firebase/common/ModLogger.dart';
import 'package:login_firebase/data/repos/SessionRepoImpl.dart';
import 'package:login_firebase/models/ThirdPartySessionModel.dart';

import '../../../data/exception/ModDataException.dart';


const String TAG = "BaseSigninBloc";


/// Base business logic component for any signin mechanism
abstract class ModBaseSigninBloc extends ModBloc<ModSessionDTO, ThirdPartySessionModel> {
  ISessionRepo repo = SessionRepoImpl();

  ModBaseSigninBloc() : super();

  @override
  void performOperation(ModSessionDTO dto) async {

    var res;
    try {
      var data = await this.callAuthentication(dto);

      // DATA
      if (data != null) {
        res = this.buildResult(data: data);

      // ERROR
      } else {
        throw Exception();
      }

    } on ModDataException catch (lde) {
      ModLogger.e(tag: TAG, msg: "${lde.toString()}");

      res = this.buildResult(data: null, code:lde.code ?? this.getErrorCode());

    } on PlatformException catch (pe) {
      ModLogger.e(tag: TAG, msg: "${pe.toString()}");

      res = this.buildResult(data: null, code: this.getErrorCode(e: pe));

    } on Exception catch (e) {
      ModLogger.e(tag: TAG, msg: "${e.toString()}");

      res = this.buildResult(data: null, code: this.getErrorCode());

    } finally {
      this.processData(res);
    }
  }

  /// Returns custom error code
  /// Must be overriden by children
  int getErrorCode({Exception? e});

  /// Launches auth flow.
  /// Must be overriden by children
  Future<ThirdPartySessionModel?> callAuthentication(ModSessionDTO dto);
}