import 'package:flutter/cupertino.dart';
import 'package:login_firebase/business/bloc/CustomSigninBloc.dart';
import 'package:login_firebase/business/dto/ModSessionDTO.dart';
import 'package:login_firebase/common/ModLogger.dart';
import 'package:login_firebase/models/ModResourceResult.dart';
import 'package:login_firebase/models/ThirdPartyErrorModel.dart';
import 'package:login_firebase/models/ThirdPartySessionModel.dart';
import 'package:login_firebase/presentation/widgets/IForm.dart';
import 'package:login_firebase/presentation/widgets/base/ModBaseStatelessWidgetWithBloc.dart';
import 'package:login_firebase/presentation/widgets/convenient/ModLoadingWidget.dart';


const String TAG = "CustomSigninWidget";

//ignore: must_be_immutable
class CustomSigninWidget extends ModBaseStatelessWidgetWithBloc<CustomSigninBloc, ThirdPartySessionModel> {
  final Widget child;
  final Function(ThirdPartySessionModel?) onSessionSuccess;
  final Function(int) onSessionError;
  final Function() onReset;


  CustomSigninWidget({
    required this.child,
    required this.onSessionSuccess,
    required this.onSessionError,
    required this.onReset,
    Key? key}) :
        super(key: key) {

    if (this.child is IForm) {
      (this.child as IForm).setListener(onChildValidated);
    } else {
      ModLogger.e(tag: TAG, msg: "Unable to set custom signin");
    }
  }

  @override
  Widget buildErrorState(BuildContext context, ThirdPartyErrorModel? error) {
    this.onSessionError(error!.code);

    return this.child;
  }

  @override
  Widget buildInitialState(BuildContext context) {
    return this.child;
  }

  @override
  Widget buildLoadingState(BuildContext context) {
    return Stack(
      children: [
        this.child,
        ModLoadingWidget()
      ],
    );
  }

  @override
  Widget buildSuccessState(BuildContext context, ThirdPartySessionModel? data) {
    if (data!.validate()) {
      this.onSessionSuccess(data);
    }

    return this.child;
  }

  @override
  void initBloc(BuildContext context) {
    this.bloc = CustomSigninBloc();
  }

  void onChildValidated(String mail, String pass) {
    this._call(mail, pass);
  }

  void _call(String mail, String pass) {
    this.bloc!.processData(ModResourceResult(state: ResourceState.LOADING, data: null, error: null));
    this.bloc!.performOperation(ModSessionDTO(mail: mail, pass: pass));
  }
}